import pygame
import data
import spider
import path
import math
import moth
import random

GAME_TITLE = "Lantern Defender, by Trobadour for PyWeek 11"
SCREEN_SIZE = ( 800, 600 )
FRAMES_PER_SECOND = 30.0
SHOTS_PER_MOTH = 10

def load_image( file, new_size=(0,0) ):
    filename = data.filepath( file )
    try:
        surface = pygame.image.load( filename )
    except pygame.error:
        raise SystemExit, 'Could not load image "%s" %s'%( filename, pygame.get_error())
    surface = surface.convert_alpha()
    if new_size != (0,0):
        surface = pygame.transform.scale( surface, new_size )
    return surface

def target_cursor():
    return (
        "         XXXXXX         ",
        "       XX  XX  XX       ",
        "     XX    XX    XX     ",
        "    X      XX      X    ",
        "   X       XX       X   ",
        "  X        XX        X  ",
        "  X        XX        X  ",
        " X         XX         X ",
        " X         XX         X ",
        "X          XX          X",
        "X          XX          X",
        "XXXXXXXXXXX  XXXXXXXXXXX",
        "XXXXXXXXXXX  XXXXXXXXXXX",
        "X          XX          X",
        "X          XX          X",
        " X         XX         X ",
        " X         XX         X ",
        "  X        XX        X  ",
        "  X        XX        X  ",
        "   X       XX       X   ",
        "    X      XX      X    ",
        "     XX    XX    XX     ",
        "       XX  XX  XX       ",
        "         XXXXXX         ")

def distance( p1, p2 ):
    d = math.sqrt( pow(p2[0]-p1[0],2) + pow(p2[1]-p1[1],2) )
    return d

class Shot:
    def __init__( self, start_pos, end_pos ):
        self.position = start_pos
        self.radius = 3
        vector_x = end_pos[0]-start_pos[0]
        vector_y = end_pos[1]-start_pos[1]
        norm = math.sqrt( vector_x*vector_x + vector_y*vector_y )
        self.diff_x = vector_x / norm
        self.diff_y = vector_y / norm
        self.movement_speed = 2.0
        return
    
    def update( self ):
        self.position = ( self.position[0]+(self.diff_x*self.movement_speed), self.position[1]+(self.diff_y*self.movement_speed) )
        return
    
    def draw( self, surface ):
        pygame.draw.circle( surface, (128,128,128), (int(self.position[0]),int(self.position[1])), self.radius, 1 )
        return
  
class Game:
    def __init__( self ):
        pygame.init()
        pygame.display.set_caption( GAME_TITLE )
        random.seed()
        
        self.run_game = True
        self.game_over = False
        self.fullscreen = False
        self.pause = True
        self.lantern_max_health = 5
        self.lantern_current_health = 5
        self.max_moths = 1
        self.xp = 0
        self.score = 0
        self.moth_counter = 0
        self.increase_difficulty_at = 5
        self.increase_offset = 7
        self.MOTH_SPEED = 1.0
        self.MOTH_STUN_TIME = 90
        self.SHOT_SPEED = 2.0
        
        self.screen = self.set_fullscreen_mode( self.fullscreen )
        self.small_font = pygame.font.Font( None, 20 )
        self.large_font = pygame.font.Font( None, 50 )
        
        # assets
        self.path_container = path.PathContainer()
        self.spider = None
        self.background = load_image( "LizzieThomas.png", SCREEN_SIZE )
        self.shots = []
        self.moths = []
        
        data, mask = pygame.cursors.compile( target_cursor() )
        pygame.mouse.set_cursor( (24,24), (12,12), mask, data )
        
        self.init_level()
        return

    def add_moth( self ):
        # determine border
        border = random.randint( 1, 4 )
        x = 0
        y = 0
        if border is 1:
            x = random.randint( 0, SCREEN_SIZE[0] )
        if border is 2:
            y = random.randint( 0, SCREEN_SIZE[1] )
        if border is 3:
            x = random.randint( 0, SCREEN_SIZE[0] )
            y = SCREEN_SIZE[1] - 1
        if border is 4:
            x = SCREEN_SIZE[0] - 1
            y = random.randint( 0, SCREEN_SIZE[1] )
        new_moth = moth.Moth( (x,y), (430,232), load_image("moth.png",(40,25)) )
        new_moth.movement_speed = self.MOTH_SPEED
        new_moth.FRAMES_PER_STUN = self.MOTH_STUN_TIME
        self.moths.append( new_moth )
        self.moth_counter = self.moth_counter + 1
        return
    
    def grab_moth( self, cursor_position ):
        spider_position = self.spider.position
        if distance(spider_position,cursor_position) > self.spider.grab_range:
            return
        for moth in self.moths:
            if moth.stunned is 0:
                continue
            if distance(cursor_position,moth.position) <= moth.radius:
                self.moths.remove( moth )
                return True
        return False
    
    def init_level( self ):
        self.spider = spider.Spider( (325,149), load_image("spider.png",(25,25)) , self.path_container )
        self.path_container.add_path( (201,428), (323,463) )
        self.path_container.add_path( (323,463), (428,398) )
        self.path_container.add_path( (420,484), (324,463) )
        self.path_container.add_path( (324,463), (265,540) )
        self.path_container.add_path( (366,591), (326,461) )
        self.path_container.add_path( (326,461), (221,301) )
        self.path_container.add_path( (327,460), (326,148) )
        self.path_container.add_path( (327,351), (403,414) )
        self.path_container.add_path( (403,414), (395,481) )
        self.path_container.add_path( (395,481), (347,535) )
        self.path_container.add_path( (347,535), (283,522) )
        self.path_container.add_path( (283,522), (242,439) )
        self.path_container.add_path( (242,439), (260,368) )
        self.path_container.add_path( (260,368), (327,352) )
        self.path_container.add_path( (325,149), (430,184) )
        self.path_container.add_path( (326,150), (191,44) )
        self.path_container.add_path( (206,221), (325,149) )
        self.path_container.add_path( (325,149), (364,4) )
        self.path_container.add_path( (568,117), (621,56) )
        self.path_container.add_path( (674,225), (567,115) )
        self.path_container.add_path( (567,115), (428,183) )
        self.path_container.add_path( (566,115), (516,12) )
        self.path_container.add_path( (566,118), (700,116) )
        self.path_container.add_path( (567,116), (435,308) )
        self.path_container.add_path( (435,308), (545,391) )
        self.path_container.add_path( (545,391), (459,568) )
        self.path_container.add_path( (545,391), (620,369) )
        self.path_container.add_path( (546,392), (619,470) )
        self.path_container.add_path( (446,416), (547,392) )
        self.path_container.add_path( (547,392), (655,269) )
        self.path_container.add_path( (606,325), (478,338) )
        self.path_container.add_path( (478,338), (483,407) )
        self.path_container.add_path( (483,407), (498,486) )
        self.path_container.add_path( (498,486), (599,446) )
        self.path_container.add_path( (599,446), (614,373) )
        self.path_container.add_path( (614,373), (605,325) )
        self.path_container.add_path( (513,193), (476,155) )
        self.path_container.add_path( (476,155), (535,45) )
        self.path_container.add_path( (535,45), (611,68) )
        self.path_container.add_path( (611,68), (630,120) )
        self.path_container.add_path( (630,120), (616,163) )
        self.path_container.add_path( (616,163), (511,192) )
        self.path_container.add_path( (351,55), (237,81) )
        self.path_container.add_path( (237,81), (253,194) )
        self.path_container.add_path( (253,194), (326,246) )
        self.path_container.add_path( (326,246), (381,168) )
        self.path_container.add_path( (350,53), (381,165) )
        return
    
    def set_fullscreen_mode( self, fullscreen=True ):
        self.fullscreen = fullscreen
        if self.fullscreen:
            return pygame.display.set_mode( SCREEN_SIZE, pygame.FULLSCREEN )
        else:
            return pygame.display.set_mode( SCREEN_SIZE )
        return
    
    def handle_pressed_keys( self ):
        keys = pygame.key.get_pressed()
        self.spider.handle_pressed_keys( keys )
        return
    
    def handle_events( self ):
        for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.run_game = False
                elif event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_ESCAPE:
                        self.run_game = False
                    elif event.key == pygame.K_f:
                        self.fullscreen = not self.fullscreen
                        self.screen = self.set_fullscreen_mode( self.fullscreen )
                    elif event.key == pygame.K_1 and self.xp>=1:
                        self.spider.shots_left = self.spider.shots_left + 10
                        self.xp = self.xp - 1
                    elif event.key == pygame.K_2 and self.xp>=2:
                        self.spider.movement_speed = self.spider.movement_speed + 1.0
                        self.xp = self.xp - 2
                    elif event.key == pygame.K_3 and self.xp>=3:
                        self.spider.grab_range = self.spider.grab_range + 15
                        self.xp = self.xp - 3
                    elif event.key == pygame.K_4 and self.xp>=3:
                        self.MOTH_STUN_TIME = self.MOTH_STUN_TIME + 30
                        self.xp = self.xp - 3
                    elif event.key == pygame.K_5 and self.xp>=3 and self.lantern_current_health<self.lantern_max_health:
                        self.lantern_current_health = self.lantern_current_health + 1
                        self.xp = self.xp - 3
                    elif event.key == pygame.K_SPACE:
                        self.pause = not self.pause
                elif event.type == pygame.MOUSEBUTTONDOWN and self.pause is False and self.game_over is False:
                    if event.button is 1:
                        if self.spider.shots_left > 0:
                            self.spider.shots_left = self.spider.shots_left - 1
                            self.shots.append( Shot(self.spider.position,pygame.mouse.get_pos()) )
                    elif event.button is 2 or event.button is 3:
                        if self.grab_moth( event.pos ):
                            self.spider.shots_left = self.spider.shots_left + SHOTS_PER_MOTH
                            self.score = self.score + 1
                elif self.pause is False and self.game_over is False:
                    self.spider.handle_event( event )
                
        return
    
    def update( self ):
        for shot in self.shots:
            for moth in self.moths:
                if distance(shot.position,moth.position) < moth.radius:
                    if self.shots.count(shot)>0:
                        self.shots.remove( shot )
                    moth.stun()
        for shot in self.shots:
            shot.update()
            if shot.position[0]<0 or shot.position[0]>SCREEN_SIZE[0] or shot.position[1]<0 or shot.position[1]>SCREEN_SIZE[1]:
                if self.shots.count(shot)>0:
                    self.shots.remove( shot )
        for moth in self.moths:
            moth.update()
            if distance(moth.position,(430,232))<=max(abs(moth.diff_x*moth.movement_speed),abs(moth.diff_y*moth.movement_speed)):
                if self.moths.count(moth)>0:
                    self.moths.remove( moth )
                self.lantern_current_health = self.lantern_current_health - 1
        if self.moths.__len__()<self.max_moths:
            self.add_moth()
        return
    
    def draw( self ):
        self.screen.fill( (255,255,255) )
        
        # lantern
        color = int( 255 * float(self.lantern_current_health) / float(self.lantern_max_health) )
        color = max( color, 0 )
        pygame.draw.rect( self.screen, (color,color,0), ((422,219),(14,30)) )
        
        self.screen.blit( self.background, (0,0) )
        
        self.path_container.draw( self.screen )
        self.spider.draw( self.screen )
        
        for shot in self.shots:
            shot.draw( self.screen )
            
        for moth in self.moths:
            moth.draw( self.screen )
            
        if self.game_over is True:
            text = self.large_font.render( "GAME OVER", True, (255,0,0) )
            self.screen.blit( text, (SCREEN_SIZE[0]/2-text.get_rect().width/2,50) )
            
            text = self.large_font.render( "Your score is %s" % (self.score), True, (255,0,0) )
            self.screen.blit( text, (SCREEN_SIZE[0]/2-text.get_rect().width/2,170) )
        
        elif self.pause is True:
            text = self.large_font.render( "Welcome to 'Lantern Defender'", True, (255,0,0) )
            self.screen.blit( text, (SCREEN_SIZE[0]/2-text.get_rect().width/2,50) )
            
            text = self.large_font.render( "Press <SPACE> to continue", True, (255,0,0) )
            self.screen.blit( text, (SCREEN_SIZE[0]/2-text.get_rect().width/2,170) )
            
            pygame.draw.rect( self.screen, (0,0,0), ((150,300),(500,250)) )
            
            text = self.small_font.render( "Goal: Defend the lantern from incoming moths.", True, (255,0,0) )
            self.screen.blit( text, (160,310) )
            
            text = self.small_font.render( "- Don't let the lantern energy become zero!", True, (255,0,0) )
            self.screen.blit( text, (160,325) )
            
            text = self.small_font.render( "- You are the spider, the spider can only move on the net.", True, (255,0,0) )
            self.screen.blit( text, (160,340) )
            
            text = self.small_font.render( "- First shot at moths to stun them, then catch them.", True, (255,0,0) )
            self.screen.blit( text, (160,355) )
            
            text = self.small_font.render( "- You can only catch moths that are inside the catch range (circle).", True, (255,0,0) )
            self.screen.blit( text, (160,370) )
            
            text = self.small_font.render( "- Catching stunned moths gives you more shots and XP.", True, (255,0,0) )
            self.screen.blit( text, (160,385) )
            
            text = self.small_font.render( "- Use XP to buy improvements.", True, (255,0,0) )
            self.screen.blit( text, (160,400) )
            
            text = self.small_font.render( "Controls:", True, (255,0,0) )
            self.screen.blit( text, (160,420) )

            text = self.small_font.render( "<W,A,S,D> or <LEFT,RIGHT,UP,DOWN> - Move the spider.", True, (255,0,0) )
            self.screen.blit( text, (160,435) )
            
            text = self.small_font.render( "<MOUSE BUTTON 1> - Shot to stun.", True, (255,0,0) )
            self.screen.blit( text, (160,450) )
            
            text = self.small_font.render( "<MOUSE BUTTON 2> - Grab stunned moth within the catch range.", True, (255,0,0) )
            self.screen.blit( text, (160,465) )
            
            text = self.small_font.render( "<1,2,3> - Buy improvements.", True, (255,0,0) )
            self.screen.blit( text, (160,480) )
            
            text = self.small_font.render( "<SPACE> - Pause the game.", True, (255,0,0) )
            self.screen.blit( text, (160,495) )
            
            text = self.small_font.render( "<F> - Switch fullscreen.", True, (255,0,0) )
            self.screen.blit( text, (160,510) )
            
        text = self.small_font.render( "Shots left = %s" % (self.spider.shots_left), True, (255,255,255) )
        self.screen.blit( text, (10,10) )
        
        text = self.small_font.render( "Lantern Energy = %s" % (self.lantern_current_health), True, (255,255,255) )
        self.screen.blit( text, (10,20) )
        
        text = self.small_font.render( "Score = %s" % (self.score), True, (255,255,255) )
        self.screen.blit( text, (10,30) )
        
        text = self.small_font.render( "XP = %s" % (self.xp), True, (255,255,255) )
        self.screen.blit( text, (10,40) )
        
        
        
        text = self.small_font.render( "Shop:", True, (255,255,255) )
        self.screen.blit( text, (10,500) )
        
        text = self.small_font.render( "<1> - 10 Shots (1 XP)", True, (255,255,255) )
        self.screen.blit( text, (10,515) )
        
        text = self.small_font.render( "<2> - Increase Spider Speed (2 XP)", True, (255,255,255) )
        self.screen.blit( text, (10,530) )
        
        text = self.small_font.render( "<3> - Increase Catch Range (3 XP)", True, (255,255,255) )
        self.screen.blit( text, (10,545) )
        
        text = self.small_font.render( "<4> - Increase Stun Time (3 XP)", True, (255,255,255) )
        self.screen.blit( text, (10,560) )
        
        text = self.small_font.render( "<5> - Lantern Energy + 1 (max 5) (3 XP)", True, (255,255,255) )
        self.screen.blit( text, (10,575) )
        
        
        
        pygame.display.flip()
        return
    
    def increase_difficulty( self ):
        if self.moth_counter == self.increase_difficulty_at:
            self.max_moths = self.max_moths + 1
            self.increase_offset = self.increase_offset + 5
            self.increase_difficulty_at = self.moth_counter + self.increase_offset
        return
    
    def startGame( self ):
        while self.run_game:
            ticks_start = pygame.time.get_ticks()
            
            self.increase_difficulty()
            
            if self.lantern_current_health <= 0:
                self.game_over = True
            
            # handle events
            if self.pause is False and self.game_over is False:
                self.handle_pressed_keys()
            self.handle_events()
            
            # update objects
            if self.pause is False and self.game_over is False:
                self.update()
            
            # draw stuff
            self.draw()
            
            # wait a little to get FRAMES_PER_SECOND
            ticks_end = pygame.time.get_ticks()
            ticks_diff = ticks_end - ticks_start
                        
            if ticks_diff < 1000/FRAMES_PER_SECOND:
                delay = 1000/int(FRAMES_PER_SECOND)-ticks_diff
                pygame.time.delay( delay )
        return