import math
import pygame

class PathContainer:
    def __init__( self ):
        self.paths = []
        return
    
    def add_path( self, start_pos, end_pos ):
        self.paths.append( Path(start_pos,end_pos) )
        return
    
    def is_valid_spider_position( self, position ):
        for path in self.paths:
            if path.is_on_path( position ):
                return True
        return False
    
    def remove_path( self, position ):
        for path in self.paths:
            if path.is_on_path( position ):
                self.paths.remove( path )
                return
        return
    
    def create_level_file( self ):
        f = open( "level.txt", "w" )
        for path in self.paths:
            x1 = int(path.start_pos[0])
            y1 = int(path.start_pos[1])
            x2 = int(path.end_pos[0])
            y2 = int(path.end_pos[1])
            f.write( ".add_path( (%s,%s), (%s,%s) )\n" % ( x1, y1, x2, y2 ) )
        f.close()
        return
    
    def draw( self, surface ):
        for path in self.paths:
            path.draw( surface )
        return

class Path:
    def __init__( self, start_pos, end_pos ):
        self.start_pos = start_pos
        self.end_pos = end_pos
        self.visible_width = 1
        self.walkable_width = 15
        
        self.vertical = False
        self.horizontal = False

        if start_pos[0] is end_pos[1]:
            self.vertical = True
        if start_pos[1] is end_pos[1]:
            self.horizontal = True
        
        self.path_slope = 0
        self.path_offset = 0
        
        # determine walkanle bounding box
        self.min_x = min( start_pos[0], end_pos[0] ) - self.walkable_width/2
        self.min_y = min( start_pos[1], end_pos[1] ) - self.walkable_width/2
        self.max_x = max( start_pos[0], end_pos[0] ) + self.walkable_width/2
        self.max_y = max( start_pos[1], end_pos[1] ) + self.walkable_width/2
        
        # calculate line equation
        if self.vertical is False and self.horizontal is False:
            left = (0,0) 
            right = (0,0)
            if start_pos[0] < end_pos[0]:
                left = start_pos
                right = end_pos
            else:
                left = end_pos
                right = start_pos
            self.path_slope = float(end_pos[1]-start_pos[1]) / float(end_pos[0]-start_pos[0])
            self.path_offset = start_pos[1] - ( self.path_slope * start_pos[0] )
        return
    
    def draw( self, surface ):
        #pygame.draw.line( surface, (128,128,128), self.start_pos, self.end_pos, self.walkable_width )
        pygame.draw.line( surface, (0,0,0), self.start_pos, self.end_pos, self.visible_width )
        return

    def is_walkable( self ):
        return False
    
    def is_on_path( self, position ):
        
        # outside the bounding box?
        if position[0]<self.min_x or position[0]>self.max_x:
            return False
        if position[1]<self.min_y or position[1]>self.max_y:
            return False
        
        # in case of a vertical path
        if self.vertical is True:
            if abs(position[0]-self.start_pos[0]) <= self.walkable_width/2:
                return True
            else:
                return False
            
        # in case of a horizontal path        
        if self.horizontal is True:
            if abs(position[1]-self.start_pos[1]) <= self.walkable_width/2:
                return True
            else:
                return False
        
        # otherwise determine the intersection point and the according distance
        slope_2 = -1 / self.path_slope
        offset_2 = position[1] - slope_2*position[0]
        
        intersection_x = (offset_2 - self.path_offset) / (self.path_slope-slope_2)
        intersection_y = slope_2 * intersection_x + offset_2
        
        distance = math.sqrt( math.pow(intersection_x-position[0],2) + math.pow(intersection_y-position[1],2) )
        
        if distance <= self.walkable_width/2:
            return True
        else:
            return False