import pygame
import math

class Spider:
    def __init__( self, position, icon, path_container ):
        self.direction_degree = 0
        self.rotation_speed = 3
        self.movement_speed = 1.0
        self.spinning = False
        self.grab_range = 50
        self.shots_left = 10
        
        self.position = position
        self.icon = icon
        self.rotated_icon = pygame.transform.rotate( self.icon, 270-self.direction_degree )
        self.draw_position = ( int(position[0]), int(position[1]) )
        self.path_container = path_container
        self.radius = icon.get_rect().width/2
        self.new_thread_start = self.position
        return
    
    def handle_pressed_keys( self, keys ):
        if keys[pygame.K_a] or keys[pygame.K_LEFT]:
            self.direction_degree = self.direction_degree - self.rotation_speed
            self.rotated_icon = pygame.transform.rotate( self.icon, 270-self.direction_degree )
        if keys[pygame.K_d] or keys[pygame.K_RIGHT]:
            self.direction_degree = self.direction_degree + self.rotation_speed
            self.rotated_icon = pygame.transform.rotate( self.icon, 270-self.direction_degree )
        if keys[pygame.K_w] or keys[pygame.K_UP]:
            self.move_forward()
        if keys[pygame.K_s] or keys[pygame.K_DOWN]:
            self.move_backward()
        return
    
    def move_forward( self ):
        if self.spinning is False:
            if self.path_container.is_valid_spider_position( self.forward_position() ):
                self.position = self.forward_position()
        else:
            self.position = self.forward_position()
        # update draw position
        self.draw_position = ( int(self.position[0]), int(self.position[1]) )
        return
    
    def move_backward( self ):
        if self.spinning is False:
            if self.path_container.is_valid_spider_position( self.backward_position() ):
                self.position = self.backward_position()
        else:
            self.position = self.backward_position()
        # update draw position
        self.draw_position = ( int(self.position[0]), int(self.position[1]) )
        return
    
    def forward_position( self ):
        to_x = math.cos( math.radians(self.direction_degree) ) * self.movement_speed
        to_y = math.sin( math.radians(self.direction_degree) ) * self.movement_speed
        end_pos = ( self.position[0]+to_x, self.position[1]+to_y )
        return end_pos
    
    def backward_position( self ):
        to_x = math.cos( math.radians(self.direction_degree+180.0) ) * self.movement_speed
        to_y = math.sin( math.radians(self.direction_degree+180.0) ) * self.movement_speed
        end_pos = ( self.position[0]+to_x, self.position[1]+to_y )
        return end_pos
    
    def handle_event( self, event ):
        return False
        #if event.type == pygame.KEYDOWN:
            #if event.key == pygame.K_1:
                #self.spinning = True
                #self.new_thread_start = self.position
                #return True
            #if event.key == pygame.K_2:
                #self.spinning = False
                #self.path_container.add_path( self.new_thread_start, self.position )
                #return True
            #if event.key == pygame.K_2:
                #self.spinning = False
                #self.path_container.add_path( self.new_thread_start, self.position )
                #return True
            #if event.key == pygame.K_DELETE:
                #self.path_container.remove_path( self.position )
                #return True
            #if event.key == pygame.K_c:
                #self.path_container.create_level_file()
                #return True
        #return False
    
    def draw( self, surface ):
        # catch range
        pygame.draw.circle( surface, (0,0,0), self.draw_position, self.grab_range, 1 )
        upperleft = (self.draw_position[0]-self.rotated_icon.get_rect().width/2,self.draw_position[1]-self.rotated_icon.get_rect().height/2)
        lowerright = (self.draw_position[0]+self.rotated_icon.get_rect().width/2,self.draw_position[1]+self.rotated_icon.get_rect().height/2)
        surface.blit( self.rotated_icon, ( upperleft, lowerright ) )
        # direction
        to_x = math.cos( math.radians(self.direction_degree) ) * 10
        to_y = math.sin( math.radians(self.direction_degree) ) * 10
        start_pos = self.position
        end_pos = ( start_pos[0]+to_x, start_pos[1]+to_y )
        pygame.draw.line( surface, (0,0,0), start_pos, end_pos )
        
        # thread
        if self.spinning is True:
            pygame.draw.line( surface, (0,0,0), self.new_thread_start, self.position )
        
        return
    